{
	ID3V1*Tag Module for Wyvern (C)1999-2000 MinMin.
}

unit ID3TagControl;

interface

uses SysUtils, Windows, Classes;

const TAG_LENGTH = 128; TAG_LONG = 30;
      TAG_SHRT   = 4;   TAG_HD   = 'TAG';
type
    TTagArray = array of string[TAG_LONG];
    TYearTag = string[TAG_SHRT];
    TID3Tags = class(TObject)
    private
        FTitle:  string;
        FArtist: string;
        FAlbum: string;
        FYear:   string;
        FComment:string;
        FGenre: byte;
        FTrack: byte;
        FFileName: string;
        function  GetCharSet(TestStr: string; LimitCount: integer): string;
    public
        RGenre: integer;
        RTagInfo: array[0..4] of string;
        constructor Create(FileName: string);
        procedure WriteTags(TrackEnabled: boolean);
        property Title:   string  read FTitle    write FTitle;
        property Artist:  string  read FArtist   write FArtist;
        property Album:   string  read FAlbum    write FAlbum;
        property Year:    string  read FYear     write FYear;
        property Comment: string  read FComment  write FComment;
        property Genre:   byte    read FGenre    write FGenre;
        property Track:   byte    read FTrack    write FTrack;
    end;

implementation


{ TID3Tags }

constructor TID3Tags.Create(FileName: string);
begin
    FFileName := FileName;
    FTrack    := 0;
end;

function TID3Tags.GetCharSet(TestStr: string; LimitCount: integer): string;
var Cnt: integer;
begin
    Result := TestStr;
    Cnt    := Length(TestStr);
    if Cnt > LimitCount then
      Result := Copy(TestStr, 0, LimitCount)
    else
    if Cnt < LimitCount then
      Result := TestStr + StringOfChar(#32, LimitCount - Cnt);
end;

procedure TID3Tags.WriteTags(TrackEnabled: boolean);
var
    FStream: TFileStream;
    TTAG: string;
    HeadBuf: string;
    CommBuf: string;
    GENRE_POS, INIT_POS, COMM_LENGTH: integer;
    MarkByte: byte;
begin
    INIT_POS  := 0;
    GENRE_POS := 0;
    MarkByte  := 0;
    SetLength(TTAG, 3);
    FStream   := TFileStream.Create(FFileName, fmOpenReadWrite);
    try
      HeadBuf := TAG_HD +
                 GetCharSet(FTitle,  TAG_LONG) +
                 GetCharSet(FArtist, TAG_LONG) +
                 GetCharSet(FAlbum,  TAG_LONG) +
                 GetCharSet(FYear,   TAG_SHRT);
      SetLength(HeadBuf, TAG_LENGTH - 31);
      if TrackEnabled then
        COMM_LENGTH := TAG_LONG - 2
      else
        COMM_LENGTH := TAG_LONG;
      CommBuf := GetCharSet(FComment, COMM_LENGTH);
      SetLength(CommBuf, COMM_LENGTH);
      with FStream do begin
        Seek(-TAG_LENGTH, soFromEnd);
        Read(PChar(TTAG)^, 3);
        if TTAG = TAG_HD then begin
          INIT_POS  := -TAG_LENGTH;
          GENRE_POS := -1;
        end;
        Seek(INIT_POS, soFromEnd);
        WriteBuffer(PChar(HeadBuf)^, Length(HeadBuf));
        WriteBuffer(PChar(CommBuf)^, Length(CommBuf));
        if TrackEnabled then begin
          WriteBuffer(MarkByte, 1);
          WriteBuffer(FTrack, 1);
        end;
        Seek(GENRE_POS, soFromEnd);
        WriteBuffer(FGenre, 1);
      end;
    finally
      FStream.Free;
    end;
end;

end.
