//*************************************************************
//
//   Miscellaneous Function (C)1999-2001 MinMin.
//
//*************************************************************


unit MyFunc;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  ShellAPI, FileCtrl, mp3plyr;


const
  SRT_BNDRY = 12;


procedure SetTrackBarFace(ATrackBar: TTrackBar);


// Merge-Sort of TList (instead of Quick-Sort)
// selsort   : SelectSortMethod
// msort     : MergeSortMethod
// SRT_BNDRY : the number of elements in border
//
procedure selsort(var SList: TList; L, R: integer; SComp: TListSortCompare);
procedure msort(var SList: TList; L, R: integer; SComp: TListSortCompare);
procedure mgSortEx(var SList: TList; SComp: TListSortCompare);

function CopyMBCS(S: String; Index, Count: Integer): string;

function ShortToLong(ShortName: string):string;

// from Delphi-ML Log
function GetDividedString(srcStr,divStr:string;Idx:Integer): string;

function FileExistsEx(FileName: string): Boolean;
function GetFileSizeEx(FileName: string): integer;
function GetMemorySize: integer;


//var CmbFlg: Boolean;

implementation


function CopyMBCS(S: String; Index, Count: Integer): string;
begin
  Result := Copy(S, Index, Count);
  if (Index < Length(S)) and
     (StrByteType(PChar(S), Index-1) = mbTrailByte) then begin
    Result[1]  :=  #$20;
  end;
  if (0 < Length(Result)) and
     (StrByteType(PChar(Result), Length(Result)-1) = mbLeadByte) then begin
    Result[Length(Result)] := #$20;
  end;
end;

function GetDividedString(srcStr, divStr: string; Idx: Integer): string;
var Psrc, Pdiv, PEnd: PChar;
    Cnt, Lgth: integer;
begin
    Result := '';
    PEnd   := nil;
    srcStr := srcStr + divStr;
    Psrc   := PChar(srcStr);
    Pdiv   := PChar(divStr);
    Lgth   := Length(divStr);
    for Cnt := 0 to Idx do begin
      if Psrc^ = #0 then Exit;
      PEnd := Psrc;
      Psrc := StrPos(Psrc, Pdiv);
      Inc(Psrc, Lgth);
    end;
    (Psrc-Lgth)^ := #0;
    Result := String(PEnd);
end;

function ShortToLong(ShortName: string):string;
var
  SearchRec: TSearchRec;
begin
    Result := '';
    ShortName := ExpandFileName(ShortName);
    while LastDelimiter('\', ShortName) >= 3 do begin
    	if FindFirst(ShortName, faAnyFile, SearchRec) = 0 then
    	   try
    	   Result := '\' + SearchRec.Name + Result;
    	   finally FindClose(SearchRec); end
    	else
    	Result := '\' + ExtractFileName(ShortName) + Result;

    	ShortName := ExtractFilePath(ShortName);
    	SetLength(ShortName, Length(ShortName) - 1);
    end;
    Result := ShortName + Result;
end;

function GetFileSizeEx(FileName: string): integer;
var SearchRec: TSearchRec;
    FileFound: Boolean;
    IsErr, SizeEx:     integer;
    s: string;
begin
  Result := 0;
  SearchRec.ExcludeAttr := faAnyfile;
  SearchRec.FindHandle  := FindFirstFile(PChar(FileName), SearchRec.FindData);
  if SearchRec.FindHandle <> INVALID_HANDLE_VALUE then begin
    SizeEx := SearchRec.FindData.nFileSizeLow;
    IsErr  := 0;
    try
      while (IsErr <> ERROR_NO_MORE_FILES) do begin
        FileFound := FindNextFile(SearchRec.FindHandle, SearchRec.FindData);
        if not FileFound then
          IsErr := GetLastError;
        if (IsErr <> ERROR_NO_MORE_FILES) then begin
          s := String(SearchRec.FindData.cFileName);
          if (s <> '.')  and (s <>'..') then
            SizeEx := SearchRec.FindData.nFileSizeLow;
        end;
      end;
    finally
      Windows.FindClose(SearchRec.FindHandle);
    end;
    SearchRec.FindHandle := INVALID_HANDLE_VALUE;
    Result := SizeEx;
  end;
end;

function GetMemorySize: integer;
var MemStat:TMemoryStatus;
begin
  MemStat.dwLength:=sizeOf(MemStat);
  GlobalMemoryStatus(MemStat);
  Result := MemStat.dwTotalPhys div 1024 div 1024;
end;

function FileExistsEx(FileName: string): Boolean;
    function FileOpenCheck(FName: string): Boolean;
    var hFile:integer;
    begin
        hFile := FileOpen(FileName, fmOpenRead);
        try
        finally FileClose(hFile); end;
        Result := (-1 <> hFile);
    end;
begin
     Result := (FileExists(FileName)) or (FileOpenCheck(FileName));
end;

procedure selsort(var SList: TList; L, R: integer; SComp: TListSortCompare);
var idx1, idx2: integer;
begin
    for idx1 := L to R - 1 do begin
    	for idx2 := idx1 + 1 to R do begin
    	if SComp(SList[idx1], SList[idx2]) > 0 then SList.Exchange(idx1, idx2);
    	end;
    end;
end;

procedure msort(var SList: TList; L, R: integer; SComp: TListSortCompare);
var cnt1, cnt2, Z: integer; tmpList: TList;
begin
    Z := (L + R) shr 1;

    case (Z - L) of
    1:            if SComp(SList[L], SList[Z]) > 0 then SList.Exchange(L, Z);
    2..SRT_BNDRY: selsort(SList, L, Z, SComp);
    SRT_BNDRY+1..MaxInt: msort(SList, L, Z, SComp);
    //2..MaxInt: msort(SList, L, Z, SComp);
    end;

    case (R - Z - 1) of
    1:            if SComp(SList[Z + 1], SList[R]) > 0 then SList.Exchange(Z + 1, R);
    2..SRT_BNDRY: selsort(SList, Z + 1, R, SComp);
    SRT_BNDRY+1..MaxInt: msort(SList, Z + 1, R, SComp);
    //2..MaxInt: msort(SList, Z + 1, R, SComp);
    end;

    tmpList := TList.Create;
    tmpList.Capacity := R - L + 1;
    cnt1    := L; 
    cnt2    := Z + 1;
    try
    while (cnt1 <= Z) or (cnt2 <= R) do begin
        if cnt1 > Z then begin
          while cnt2 <= R do begin tmpList.Add(SList[cnt2]); Inc(cnt2); end;
          Break;
          end
        else
        if cnt2 > R then begin
          while cnt1 <= Z do begin tmpList.Add(SList[cnt1]); Inc(cnt1); end;
          Break;
          end
        else
            begin
            if SComp(SList[cnt1], SList[cnt2]) > 0 then begin
                tmpList.Add(SList[cnt2]); Inc(cnt2); end
            else begin
                tmpList.Add(SList[cnt1]); Inc(cnt1); end;
            end;
        end;
    for cnt1 := 0 to tmpList.Count - 1 do
    SList[L + cnt1] := tmpList[cnt1];
    finally tmpList.Free; end;
end;

procedure mgSortEx(var SList: TList; SComp: TListSortCompare);
begin
    if SList.Count > 1 then msort(SList, 0, SList.Count - 1, SComp);
end;

procedure SetTrackBarFace(ATrackBar: TTrackBar);
begin
    if Assigned(ATrackBar) then
    SetWindowLong(ATrackBar.Handle, GWL_STYLE,
          GetWindowLong(ATrackBar.Handle, GWL_STYLE) and (not $0020));
end;


end.
