{
    ScrollLabel for Wyvern
}

unit ScrollLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, TimerEv;

const 
  SCRL_INTV = 110;
  SCRL_STEP = 3;
  //SCRL_WAIT = 300;

type
  TScrollLabel = class(TLabel)
  private
    { Private 錾 }
    FTimer: TTimerEv;
    FBmp:   TBitmap;
    FTWidth: Word;
    FPos: integer;
    FInterval: Word;
    FStep: Word;
    FText: string;
    FEdgeLoop: boolean;
    //FWaitTime: UINT;
    procedure TimerFree;
    procedure SetText(const Value: string);
    procedure TimerInitialize;
    procedure BitmapInitialize;
    procedure SetBitmap;
    procedure FTimerTimer(Sender: TObject);
    procedure SetInterval(const Value: Word);
    procedure WMSize(var Msg: TMessage); message WM_SIZE;
    procedure WMFontChange(var Msg: TWMFontChange); message WM_FONTCHANGE;
    function GetTimerEnabled: boolean;
  protected
    { Protected 錾 }
  public
    { Public 錾 }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure Scroll;
    procedure UnScroll;
    procedure ReSetFont;
    procedure ClearText;
    procedure SetCaptionString(str: string);
    property Text: string read FText write SetText;
    property IsTimerEnabled: boolean read GetTimerEnabled;
  published
    { Published 錾 }
    property Interval: Word read FInterval write SetInterval default SCRL_INTV;
    property Step:     Word read FStep     write FStep       default SCRL_STEP;
    property EdgeLoop: boolean read FEdgeLoop write FEdgeLoop default true;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TScrollLabel]);
end;

{ TScrollLabel }

constructor TScrollLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Canvas.Brush.Style := bsClear;
  FTimer := nil;
  FBmp   := nil;
  FText  := '';
  FInterval := SCRL_INTV;
  FStep     := SCRL_STEP;
  //FWaitTime := SCRL_WAIT;
  FPos      := Width;
  FEdgeLoop := true;
end;

destructor TScrollLabel.Destroy;
begin
  TimerFree;
  inherited Destroy;
end;

procedure TScrollLabel.FTimerTimer(Sender: TObject);
begin
  with Canvas do begin
    Dec(FPos, FStep);
    if (FPos + FTWidth) <= 0 then begin
      if FEdgeLoop then
        FPos := Width
      else
        FPos := 0;
    end;
    Draw(FPos, 0, FBmp);
  end;
end;

procedure TScrollLabel.TimerInitialize;
begin
  if not Assigned(FTimer) then begin
    FTimer := TTimerEv.Create(Self);
    FTimer.Enabled := false;
    FTimer.Interval := FInterval;
    Ftimer.OnTimer  := FTimerTimer;
  end;
end;

procedure TScrollLabel.BitmapInitialize;
begin
  if not Assigned(FBmp) then begin
    FBmp := TBitmap.Create;
  end;
end;

procedure TScrollLabel.Scroll;
begin
  //if FText <> '' then exit;
  TimerInitialize;
  BitmapInitialize;
  FTimer.Enabled := false;
  SetBitmap;
  FPos   := 0;
  FTimer.Enabled := true;
end;

procedure TScrollLabel.SetBitmap;
var y: integer;
begin
  if Assigned(FBmp) then begin
    FBmp.Height := Height;
    with FBmp.Canvas do begin
      Brush.Color := Color;
      FillRect(ClipRect);
    end;
    FBmp.Canvas.Font.Assign(Font);
    FTWidth := FBmp.Canvas.TextWidth(FText);
    Inc(FTWidth, (Font.Size div 2));
    FBmp.Width := FTWidth;
    y := (Height - FBmp.Canvas.TextHeight(FText)) div 2;
    FBmp.Canvas.TextOut(0, y, FText);
  end;
end;

procedure TScrollLabel.SetInterval(const Value: Word);
begin
  if FInterval <> Value then begin
    FInterval := Value;
    if Assigned(FTimer) then
      FTimer.Interval := FInterval;
  end;
end;

procedure TScrollLabel.SetText(const Value: string);
begin
  if Assigned(FTimer) then
    FTimer.Enabled := false;
  FText := Value;
  if Assigned(FTimer) then begin
    SetBitmap;
    FTimer.OnTimer := FTimerTimer;
    FTimer.Enabled := true;
  end;
end;

procedure TScrollLabel.TimerFree;
begin
  if Assigned(FTimer) then begin
    FTimer.Free;
    FTimer := nil;
  end;
  if Assigned(FBmp)   then begin
    FBmp.Free;
    FBmp := nil;
  end;
end;

procedure TScrollLabel.UnScroll;
begin
  if Assigned(FTimer) then begin
    FTimer.Enabled := false;
    TimerFree;
  end;
end;

procedure TScrollLabel.SetCaptionString(str: string);
var y: integer;
begin
  UnScroll;
  with Canvas do begin
    ClearText;
    Canvas.Font.Assign(Font);
    if str <> '' then begin
      y := (Height - Canvas.TextHeight(str)) div 2;
      Canvas.TextOut(0, y, str);
    end;
  end;
  //Caption := str;
end;

procedure TScrollLabel.WMSize(var Msg: TMessage);
begin
  ReSetFont;
  if not Assigned(FTimer) then begin
    SetCaptionString(FText);
  end;
end;

procedure TScrollLabel.ReSetFont;
begin
  if Assigned(FTimer) then begin
    FTimer.Enabled := false;
    ClearText;
    SetBitmap;
    FPos := 0;
    FTimer.Enabled := true;
  end;
end;

procedure TScrollLabel.ClearText;
begin
  with Canvas do begin
    Brush.Color := clBtnFace;
    FillRect(ClientRect);
  end;
end;

procedure TScrollLabel.WMFontChange(var Msg: TWMFontChange);
begin
  ReSetFont;
end;

function TScrollLabel.GetTimerEnabled: boolean;
begin
  Result := Assigned(FTimer);
end;

end.
