//****************************************
//  Window Ȃ Timer Component
//  from Delphi}KW vol.16
//****************************************

unit TimerEv;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TTimerEv = class(TComponent)
  private
    FTimerID: DWORD;
    FEnabled: boolean;
    FInterval: Cardinal;
    FOnTimer: TNotifyEvent;
    procedure UpdateTimer;
    procedure SetEnabled(const Value: boolean);
    procedure SetInterval(const Value: Cardinal);
    procedure SetOnTimer(const Value: TNotifyEvent);
    { Private 錾 }
  protected
    { Protected 錾 }
    procedure DoTimer; virtual;
  public
    { Public 錾 }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published 錾 }
    property Enabled: boolean read FEnabled write SetEnabled default true;
    property Interval: Cardinal read FInterval write SetInterval default 1000;
    property OnTimer: TNotifyEvent read FOnTimer write SetOnTimer;
  end;

procedure Register;

implementation

var
  TimerList: TList;

procedure TimerProc(Handle: HWND; UMsg: UINT;
    idEvent: UINT; dwTime: DWORD); stdcall;
var
  i: integer;
  Timer: TTimerEv;
begin
  for i := 0 to TimerList.Count - 1 do begin
    Timer := TTimerEv(TimerList[i]);
    if Timer.FTimerID = idEvent then
      Timer.DoTimer;
  end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TTimerEv]);
end;

{ TTimerEv }

constructor TTimerEv.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEnabled  := true;
  FInterval := 1000;
end;

destructor TTimerEv.Destroy;
begin
  SetEnabled(false);
  inherited Destroy;
end;

procedure TTimerEv.DoTimer;
begin
  if Assigned(FOnTimer) then
    FOnTimer(Self);
end;

procedure TTimerEv.SetEnabled(const Value: boolean);
begin
  if FEnabled <> Value then begin
    FEnabled := Value;
    UpdateTimer;
  end;
end;

procedure TTimerEv.SetInterval(const Value: Cardinal);
begin
  if FInterval <> Value then begin
    FInterval := Value;
    UpdateTimer;
  end;
end;

procedure TTimerEv.SetOnTimer(const Value: TNotifyEvent);
begin
  FOnTimer := Value;
  UpdateTimer;
end;

procedure TTimerEv.UpdateTimer;
begin
  KillTimer(0, FTimerID);
  FTimerID := 0;
  TimerList.Remove(Self);
  if (FInterval <> 0) and
     (FEnabled) and
     (Assigned(FOnTimer)) then begin
    FTimerID := SetTimer(0, 0, FInterval, @TimerProc);
    if FTimerID <> 0 then
      TimerList.Add(Self)
    else
      raise Exception.Create('Cannot Create Timer');
  end;
end;

initialization
  TimerList := TList.Create;
finalization
  TimerList.Free;

end.
 