unit ToolbarManagerEx;
{
  author: Wolfy 2001/02/12
  version: 0.2
  license: BSD

  modified by MinMin  Jul.2001
  ToolBarInfo ...
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  toolbarcustomfrm,ComCtrls,inifiles;

type

  TToolBarInfo = record
    Name: string;
    AAToolBar: TToolBar;
    AASubButtons: TList;
    ADefaultButtons: TStrings;
    ADefaultCount: integer;
  end;

  TToolbarManager = class(TComponent)
  private
    { Private 錾 }
    FForm: TfrmToolbarCustom;
    //FToolBar: TToolBar;
    FNotButtons: TList;
    FIniFilename: String;
    FAutoSave: boolean;
    FTBInfoArray: array of TToolBarInfo;
    function WithinRange(Idx: integer): boolean;
  protected
    { Protected 錾 }
  public
    { Public 錾 }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute(Idx: integer): Boolean;
    procedure SaveDefault(Idx: integer);
    procedure Save(Idx: integer; Default: Boolean = False);
    procedure Load(Idx: integer; Default: Boolean = False);
    procedure Init(Idx: integer);
    function Add(AName: string; AToolBar: TToolBar): boolean;
    function AddNotButton(Sender: TObject): integer;
    function IndexOf(AName: string): integer;
    function NotButtonsInclude(Sender: TObject): boolean;
  published
    { Published 錾 }
    //property ToolBar: TToolBar read FToolBar write FToolBar;
    property IniFilename: String read FIniFilename write FIniFilename;
    property AutoSave: boolean read FAutoSave write FAutoSave default true;
  end;



procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Win32', [TToolbarManager]);
end;

const
  SEPA = 'Separator';

{ TToolbarManager }


function TToolbarManager.WithinRange(Idx: integer): boolean;
begin
  Result := false;
  if (Idx in [Low(FTBInfoArray)..High(FTBInfoArray)]) and
     (Length(FTBInfoArray) > 0) then
    Result := true;
end;

function TToolbarManager.IndexOf(AName: string): integer;
var Cnt: integer;
begin
  Result := -1;
  if Length(FTBInfoArray) = 0 then exit;
  for Cnt := Low(FTBInfoArray) to High(FTBInfoArray) do begin
    if FTBInfoArray[Cnt].Name = AName then begin
      Result := Cnt;
      exit;
    end;
  end;
end;

function TToolbarManager.Add(AName: string; AToolBar: TToolBar): boolean;
var NameStr: string;
    Cnt: integer;
    btn: TToolButton;
begin
  Result := false;
  if not Assigned(AToolBar) then exit;

  SetLength(FTBInfoArray, Length(FTBInfoArray) + 1);
  if AName = '' then
    NameStr := AToolBar.Name
  else
    NameStr := AName;
  with FTBInfoArray[High(FTBInfoArray)] do begin
    FTBInfoArray[High(FTBInfoArray)].Name := NameStr;
    AAToolBar    := AToolBar;
    AASubButtons := TList.Create;
    ADefaultButtons := TStringList.Create;

    ADefaultCount := AAToolBar.ButtonCount;
    if ADefaultCount > 0 then begin
      for Cnt := 0 to AAToolBar.ButtonCount - 1 do begin
        btn := AAToolBar.Buttons[Cnt];
        if (not NotButtonsInclude(btn)) and
           ((btn.Style = tbsSeparator) or
            (btn.Style = tbsDivider)) then
          //؂͖OۑȂ
          ADefaultButtons.Add(SEPA)
        else
          ADefaultButtons.Add(btn.Name);
        {if (btn.Style = tbsSeparator) or (btn.Style = tbsDivider) then
          //؂͖OۑȂ
          ADefaultButtons.Add(SEPA)
        else
          ADefaultButtons.Add(btn.Name);}
      end;
    end;
  end;
  Result := true;
end;

constructor TToolbarManager.Create(AOwner: TComponent);
begin
  inherited;
  //FSubButtons := TList.Create;
  FAutoSave   := true;
  SetLength(FTBInfoArray, 0);
  FNotButtons := TList.Create;
end;

destructor TToolbarManager.Destroy;
var Cnt: integer;
begin
  //FSubButtons.Free;
  if Length(FTBInfoArray) > 0 then begin
    for Cnt := Low(FTBInfoArray) to High(FTBInfoArray) do begin
      FTBInfoArray[Cnt].AASubButtons.Free;
      FTBInfoArray[Cnt].ADefaultButtons.Free;
    end;
  end;
  FNotButtons.Free;
  inherited;
end;

function TToolbarManager.Execute(Idx: integer): Boolean;
begin
  Result := False;

  if not WithinRange(Idx) then exit;

  //s
  if Assigned(FForm)
    //or (not Assigned(FToolBar))
     then
    Exit;

  FForm := TfrmToolbarCustom.Create(Self);
  try
    {Result := FForm.Execute(FToolBar,
                            FSubButtons,
                            FIniFilename,
                            FAutoSave);}
    Result := FForm.Execute(
              FTBInfoArray[Idx].Name,
              FTBInfoArray[Idx].AAToolBar,
              FTBInfoArray[Idx].AASubButtons,
              FIniFilename);
              //FAutoSave);
  finally
    FForm.Release;
    FForm := nil;
  end;
end;

procedure TToolbarManager.Init(Idx: integer);
//
var
  i: Integer;
  btn: TToolButton;
begin
  {if not Assigned(FToolBar) then
    Exit;}

  if not WithinRange(Idx) then exit;

  {for i := FToolBar.ButtonCount - 1 downto 0 do
  begin
    btn := FToolBar.Buttons[i];
    btn.ShowHint := True;
    //\̓Tu
    if (not btn.Visible) or (btn.Index = -1) then
    begin
      //btn.Parent := nil;
      FSubButtons.Add(btn);
    end;
  end;}

  with FTBInfoArray[Idx] do begin
    for i := AAToolBar.ButtonCount - 1 downto 0 do begin
      btn := AAToolBar.Buttons[i];
      btn.ShowHint := true;
      if (not btn.Visible) then begin
        btn.Parent := nil;
        AASubButtons.Add(btn);
      end;
    end;
  end;

  if FAutoSave then
    Save(Idx, True);
end;

procedure TToolbarManager.Load(Idx: integer; Default: Boolean = False);
//Ԃёւ
var
  i,j,mcount: Integer;
  ini: TMemIniFile;
  btn: TToolButton;
  oldlist,newlist: TList;
  name,section: String;
  dummy: TToolBar;
begin
  if not WithinRange(Idx) then exit;

  {if not Assigned(FToolBar) then
    Exit
  else}
  if not FileExists(FIniFilename) then
    Exit;

  ini := TMemIniFile.Create(FIniFilename);
  oldlist := TList.Create;
  newlist := TList.Create;
  dummy := TToolBar.Create(nil);
  try try
    {if Default then
      section := Self.Name + '_default'
    else
      section := Self.Name;}

    if Default then
      section := FTBInfoArray[Idx].Name + '_default'
    else
      section := FTBInfoArray[Idx].Name;

    //section check
    if not ini.SectionExists(section) then
      Exit;

    dummy.Visible := False;
    dummy.Enabled := False;

    //dummy.Parent := FToolBar.Parent;
    dummy.Parent := FTBInfoArray[Idx].AAToolBar.Parent;


    //SẴ{^
    {for i := 0 to FToolBar.ButtonCount - 1 do
      oldlist.Add(FToolBar.Buttons[i]);

    for i := 0 to FSubButtons.Count - 1 do
      oldlist.Add(FSubButtons[i]);}
      
    for i := 0 to FTBInfoArray[Idx].AAToolBar.ButtonCount - 1 do
      oldlist.Add(FTBInfoArray[Idx].AAToolBar.Buttons[i]);

    for i := 0 to FTBInfoArray[Idx].AASubButtons.Count - 1 do
      oldlist.Add(FTBInfoArray[Idx].AASubButtons[i]);

    //parentς
    for i := oldlist.Count - 1 downto 0 do
    begin
      btn := oldlist[i];
      btn.Parent := nil;
      btn.Visible := False;
      //؂͍폜
      if (not NotButtonsInclude(btn)) and
         ((btn.Style = tbsSeparator) or
          (btn.Style = tbsDivider)) then
      begin
        btn.Free;
        oldlist.Delete(i);
      end;
    end;

    //CJEg
    mcount := ini.ReadInteger(section,'maincount',0);
    //C
    for i := mcount - 1 downto 0 do
    begin
      name := ini.ReadString(section,'main' + IntToStr(i),'');
      //؂
      if name = SEPA then
      begin
        //쐬

        //btn := TToolButton.Create(FToolBar.Owner);
        btn := TToolButton.Create(FTBInfoArray[Idx].AAToolBar.Owner);

        btn.Parent := nil;
        btn.Style := tbsSeparator;
        btn.Width := 8;
        btn.Visible := True;
        //
        newlist.Add(btn);
      end
      else if name <> '' then //T
      begin
        for j := oldlist.Count - 1 downto 0 do
        begin
          btn := oldlist[j];
          if btn.Name = name then //
          begin
            newlist.Add(btn);
            oldlist.Delete(j);
            Break;
          end;
        end;
      end; 
    end;

    //\
    for i := 0 to newlist.Count - 1 do//wnto 0 do
    begin
      btn := newlist[i];
      btn.Visible := True;
      btn.Parent := dummy;

      //btn.Parent := FToolBar;
      btn.Parent := FTBInfoArray[Idx].AAToolBar

    end;

    {//Ȃ̂̓Tu
    FSubButtons.Clear;
    for i := 0 to oldlist.Count - 1 do
      FSubButtons.Add(oldlist[i]);}
    FTBInfoArray[Idx].AASubButtons.Clear;
    for i := 0 to oldlist.Count - 1 do
      FTBInfoArray[Idx].AASubButtons.Add(oldlist[i]);

  except
  end;
  finally
    dummy.Free;
    newlist.Free;
    oldlist.Free;
    ini.Free;
  end;

end;

procedure TToolbarManager.Save(Idx: integer; Default: Boolean = False);
//Ԃۑ Cł悵
var
  ini: TMemIniFile;
  i,mcount: Integer;
  btn: TToolButton;
  section: String;
begin

  if not WithinRange(Idx) then exit;

  {if not Assigned(FToolBar) then
    Exit;

  {if Default then
    section := Self.Name + '_default'
  else
    section := Self.Name;}
  if Default then
    section := FTBInfoArray[Idx].Name + '_default'
  else begin
    section := FTBInfoArray[Idx].Name;
  end;

  try
    ini := TMemIniFile.Create(FIniFilename);
    try
      mcount := 0;
      //CL^

      //for i := 0 to FToolBar.ButtonCount - 1 do
      for i := 0 to FTBInfoArray[Idx].AAToolBar.ButtonCount - 1 do

      begin

        //btn := FToolBar.Buttons[i];
        btn := FTBInfoArray[Idx].AAToolBar.Buttons[i];

        //؂

        if (not NotButtonsInclude(btn)) and
           ((btn.Style = tbsSeparator) or
            (btn.Style = tbsDivider)) then
          //؂͖OۑȂ
          ini.WriteString(section,'main' + IntToStr(i),SEPA)
        else begin
          ini.WriteString(section,'main' + IntToStr(i),btn.Name);
        end;
        //₷
        Inc(mcount);
      end;
      //JEg
      ini.WriteInteger(section,'maincount',mcount);
      ini.UpdateFile;
    finally
      ini.Free;
    end;
  except
  end;

end;

procedure TToolbarManager.SaveDefault(Idx: integer);
var
  ini: TMemIniFile;
  i: Integer;
  section: String;
begin
  if (not WithinRange(Idx)) or
     (not FileExists(FIniFilename)) or
     (FTBInfoArray[Idx].ADefaultButtons.Count = 0) then exit;
  section := FTBInfoArray[Idx].Name + '_default';
  try
    ini := TMemIniFile.Create(FIniFilename);
    try
      //ftHgL^
      for i := 0 to FTBInfoArray[Idx].ADefaultButtons.Count - 1 do begin
        ini.WriteString(section,'main' + IntToStr(i),
            FTBInfoArray[Idx].ADefaultButtons.Strings[i]);
      end;
      //JEg
      ini.WriteInteger(section,'maincount',FTBInfoArray[Idx].ADefaultCount); 
      ini.UpdateFile;
    finally
      ini.Free;
    end;
  except
  end;
end;

function TToolbarManager.AddNotButton(Sender: TObject): integer;
begin
  Result := FNotButtons.IndexOf(Sender);
  if Result = -1 then begin
    FNotButtons.Add(Sender);
    Result := FNotButtons.Count - 1;
  end;
end;

function TToolbarManager.NotButtonsInclude(Sender: TObject): boolean;
begin
  Result := (FNotButtons.IndexOf(Sender) <> -1);
end;

end.
 
